
var WELL_ALPHA = [ 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'];

var GENERIC_STRATAGENE_NAME = 'Stratagene.r';
var EXTERNAL_RNA_NAME = 'External.r';

/**
	Set contents of the tag with given id.
*/
function setInnerHTML(id, html)
{
	var tag = document.getElementById(id);
	if (!tag)
	{
		alert('No tag with id='+id);
	}
	else
	{
		tag.innerHTML = html;
	}
}

/**
	Execute the given method when the TAB key is pressed. A short
	delay is added so the method doesn't interfer with normal event
	handling.
	
	@param event The event object with information about what happened
	@param method The method to execute when TAB is pressed
*/
function doOnTab(event, method)
{
	if (event.keyCode == 9) 
	{
		setTimeout(method, 200);
	}
	return true;
}

/**
	Execute the given method when the ENTER key is pressed. A short
	delay is added so the method doesn't interfer with normal event
	handling.
	
	@param event The event object with information about what happened
	@param method The method to execute when ENTER is pressed
*/
function doOnEnter(event, method)
{
	if (event.keyCode == 13) 
	{
		setTimeout(function() { method.call(null, event) }, 200);
	}
	return true;
}

/**
	Execute the given method when the TAB or ENTER key is pressed. 
	A short delay is added so the method doesn't interfer with normal 
	event handling.
	
 	@param event The event object with information about what happened
	@param method The method to execute when TAB is pressed
 */
function doOnTabOrEnter(event, method)
{
	if (event.keyCode == 9 || event.keyCode == 13) 
	{
		setTimeout(method, 200);
	}
	return true;
}

/**
	Switch focus to the given input field if the ENTER key is pressed.
	If no form name is given, the input field must be on the same form
	as the field that receives the event.
	
 	@param event The event object with information about what happened
 	@param inputFieldName The name of the input field which should get focus next
*/
function focusOnEnter(event, inputFieldName, formName)
{
	if (event.keyCode == 13) 
	{
		var frm;
		if (formName)
		{
			frm = document.forms[formName];
		}
		else
		{
			var currentField = event.target || event.srcElement;
			if (currentField) frm = currentField.form;
		}
		if (frm)
		{
			var nextField = frm[inputFieldName];
			if (!nextField) nextField = document.getElementById(inputFieldName);
			if (nextField) nextField.focus();
		}
	}
	return true;
}

/**
	Set status for an input field.
	@param prefix The prefix to use for finding <div> and other tags
	@param message The message (may be empty)
	@param clazz The message clazz (valid, invalid, warning) or empty
*/
function setInputStatus(prefix, message, clazz)
{
	var statusTag = document.getElementById(prefix + '.status');
	statusTag.className = 'status ' + clazz;
	
	if (message == null) return;
	var msgTag = document.getElementById(prefix + '.message');
	if (msgTag)
	{
		setInnerHTML(prefix + '.message', message);
		if (message)
		{
			Main.showInline(prefix + '.message');
		}
		else
		{
			Main.hide(prefix + '.message');
		}
	}
	else
	{
		statusTag.title = message;
	}
}

function showLoadingAnimation(msg)
{
	var loadingDiv = document.getElementById('loading');
	if (loadingDiv) Main.show('loading');
	var loadingMsg = document.getElementById('loading.msg');
	if (loadingMsg) 
	{
		loadingMsg.innerHTML = msg ? msg : 'Please wait...';
	}
}

function hideLoadingAnimation()
{
	var ld = document.getElementById('loading');
	if (ld) Main.hide('loading');
}


/**
	Display a fatal error message and disable all form controls and
	hide all buttons except the 'gorestart' button. A <div> tag with
	the id=errorMessage is assumed.
	
	@param message The error message to display
*/
function setFatalError(message)
{
	setInnerHTML('errorMessage', message);
	Main.show('errorMessage');
	var buttons = Main.getElementsByTagAndClassName('div', 'buttonclass');
	if (buttons.length == 0) buttons = Main.getElementsByTagAndClassName('div', 'basicbutton');
	for (var i = 0; i < buttons.length; i++)
	{
		if (buttons[i].id) buttons[i].style.display = 'none';
	}
	Main.show('gorestart');
	for (var fi = 0; fi < document.forms.length; fi++)
	{
		var frm = document.forms[fi];
		for (var ei = 0; ei < frm.elements.length; ei++)
		{
			frm.elements[ei].disabled = true;	
		}
	}
}

/**
	Cancel the current registration and re-start the wizard from the beginning.
	@param force If set, the wizard is re-started without confirmation
*/
function goRestart(force)
{
	if (!force && !confirm('Cancel this registration?')) return;
	var url = location.href;
	if (url.indexOf('&restart=1') == -1) url += '&restart=1';
	location.href = url;
}

/**
	Check the control digit of the given personal number.
	@pnr The 10-digit personal number in the form YYMMDDXXXX
*/
function personalNumberControlDigitCheck(pnr)
{
	var sum = 0;
	var factor = 2;
	for (var i = 0; i < 9; i++)
	{
		var digit = parseInt(pnr.substr(i, 1), 10);
		var tmp = factor * digit;
		sum += tmp >= 10 ? tmp - 9: tmp;
		factor = factor == 2 ? 1 : 2;
	}
	
	var control = 10 - (sum % 10);
	if (control == 10) control = 0;
	return (control == parseInt(pnr.substr(9, 1), 10))
}

/**
	Convert a 2-digit year to a 4-digit year. Assuming
	that the short year is between 1 and 100 years old. 
*/
function fullYear(shortYear)
{
	var thisYear = new Date().getFullYear();
	var fullYear = shortYear + 100*Math.floor((thisYear - shortYear - 1) / 100);
	return fullYear;
}

/**
	Autofill a date-string.
	The supplemented date-string will have the format yyyyMMdd when it's returned.
	If the short-date in format MMdd is larger the current date, the year before will be used.
	
	@param shortDate Date in format 'MMdd' or in 'yyMMdd'
	@param allowSixDigitDates Boolean Flag indicating if 6-digit dates should be auto-filled (default = false).
*/
function autoFillDate(shortDate, allowSixDigitDates)
{
	if (allowSixDigitDates == null)
	{
		allowSixDigitDates = false;
	}
	shortDate = shortDate.replace("-","")
	var fullDateString = shortDate;
	
	var currentDate = new Date();
	var currentYear = currentDate.getFullYear();
	if (shortDate.length == 4 && Dates.isDate(shortDate, 'MMdd'))
	{
		var month = parseInt(shortDate.substring(0,2), 10);
		var date = parseInt(shortDate.substring(2), 10);
		if ((month > (currentDate.getMonth()+1))  ||
				( (month == (currentDate.getMonth()+1)) && (date > currentDate.getDate())) )
		{
			currentYear--;
		}
		fullDateString = currentYear+shortDate;
	}
	else if (allowSixDigitDates && shortDate.length == 6 && Dates.isDate(shortDate, 'yyMMdd'))
	{
		var year = parseInt(shortDate.substring(0,2), 10);		
		fullDateString = ((2000+year)>currentYear ? "19" : "20") + fullDateString;
	}	
	return fullDateString;	
}

function autoFillTime(shortTime)
{
	
	var fullTimeString = shortTime;
	if (shortTime.length==3) 
	{
		fullTimeString = "0"+shortTime;
	}
	return fullTimeString;
}

function getWeekByDate(currentDate)
{
	var weeknum;	
	var newYear = new Date(currentDate.getFullYear(),0,1);
	var numdays = Math.ceil((currentDate.getTime() - newYear.getTime()) / (1000 * 60 * 60 * 24));
	var day = newYear.getDay()-1;
	day = day>=0 ? day : day+7;
	var offset = day < 3 ? day : day-7;
	
	if ( (numdays+offset) > 0 )
	{
		weeknum = Math.ceil((numdays + offset) / 7);
	}
	else
	{
		// Current week belongs to the year before.
		weeknum = getWeekByDate(new Date(currentDate.getFullYear()-1, 11, 31));
	}
			
	// If the last day of the year is on a monday or thuesday. 
	if (weeknum > 52)
	{
		var nYear = new Date(newYear.getFullYear()+1, 0, 1);
		var nday = nYear.getDay()-1;
		nday = nday >=0 ? nday : nday+7;
		weeknum = nday < 4 ? 1 : 53;
	}
	return weeknum;
}

/*
	Verify that the 'case' name is a 7-digit value + optional 'C' suffix
 */
function isValidCaseName(caseName, allowSuffix)
{
	var regex = allowSuffix ? /^\d{7}C?$/ : /^\d{7}$/;
	return caseName.match(regex);
}

/*
	Verify that the RCCID number is a 10-digit value + 'B', 'C', or 'D' suffix
 */
function isValidRccidNumber(rccidNumber, allowSuffix)
{
	var regex = allowSuffix ? /^\d{10}(B|C|D){1}$/ : /^\d{10}$/;
	return rccidNumber.match(regex);
}

/**
	Get the selected value from a selection list.
*/
function getSelectedValue(list)
{
	if (list.selectedIndex < 0) return '';
	return list[list.selectedIndex].value;
}
/**
	Select the first option in the list that has the given value.
*/
function setSelectedValue(list, value)
{
	for (var i = 0; i < list.length; i++)
	{
		if (list[i].value == value)
		{
			list.selectedIndex = i;
			return;
		}
	}
}

function setCurrentStep(stepNo)
{
	var s = 1;
	while (true)
	{
		var stepDiv = document.getElementById('step.'+s);
		if (!stepDiv) break;
		
		if (s < stepNo)
		{
			stepDiv.className = 'step past';
			stepDiv.onclick = toggleStepSection(s);
			stepDiv.title = 'Show/hide this section';
		}
		else if (s == stepNo)
		{
			stepDiv.className = 'step current';
			stepDiv.onclick = null;
			stepDiv.title = null;
		}
		else
		{
			stepDiv.className = 'step future';
			stepDiv.onclick = null;
			stepDiv.title = null;
		}
		s++;
	}
}

function toggleStepSection(stepNo)
{
	return function() 
	{
		Main.showHide('step.' + stepNo + '.section') 
	};
}

function formatNumber(number, unit, baseDecimals)
{
	if (number == null) return '';
	
	var numDecimals;
	if (!baseDecimals) baseDecimals = 3;
	if (number < 10)
	{
		numDecimals = baseDecimals;
	}
	else if (number < 100)
	{
		numDecimals = baseDecimals-1;
	}
	else
	{
		numDecimals = baseDecimals-2;
	}

	var result = number.toFixed(numDecimals);
	if (unit) result += unit;
	return result;
}

function formatText(text)
{
	if (text == null) return '';
	text = text.replace(/\&/g, '&amp;');
	text = text.replace(/\</g, '&lt;');
	text = text.replace(/\>/g, '&gt;');
	text = text.replace(/\n/g, '<br>');
	return text;
}

function scrollToBottom(element)
{
	element.scrollTop = element.scrollHeight;
}

/**
	Open a popup window and copy a part of the current window to the popup
	and make it suitable for printin.

	@param ID The current session id
	@param printElementId The id of a html tag in the current page that should
		be copied to the print window
	@param pageTitle The title of the print window
	@param pageOrientation 'landscape' or 'portrait' (default)
	@param printNote Optional note intended to include instructions how to configure
		the browser for optimal printing (eg. set to landscape and scale to 75%)
*/
function openPrintWindow(ID, printElementId, pageTitle, pageOrientation, printNote, reggieRoot, extraStyles)
{
	// Default width/height is for 'portrait' orientation
	var width = 900;
	var height = 900;
	if (pageOrientation == 'landscape')
	{
		width = 1300;
		height = 700;
	}
	
	// Create callback function for the copy
	window.finalizePrint = function()
		{
			// Set page title...
			if (pageTitle) printWin.document.title = pageTitle;
		
			// ...orientation...
			if (pageOrientation)
			{
				var paper = printWin.document.getElementById('paper');
				paper.className += ' ' + pageOrientation;
			}
			
			// ... and print note
			if (printNote)
			{
				var note = printWin.document.getElementById('printNote');
				note.innerHTML = printNote;
			}
						
			// Get the HTML from the <div> tag to be printed
			var printElement = document.getElementById(printElementId);
			var printHtml = printElement.innerHTML;
			// Replace all <canvas> elements with <img>
			// since that seems the only way to make things work in IE
			printHtml = printHtml.replace(/\<canvas.*?\<\/canvas\>/g, '<img class="canvas-copy">');
		
			// Copy the HTML to the print-area in the print-window
			var printArea = printWin.document.getElementById('printarea');
			printArea.innerHTML = printHtml;
		
			// Copy the <canvas> elements as images to the <img> elements
			// using toDataURL()
			var srcCanvas = printElement.getElementsByTagName('canvas');
			var printImg = printArea.getElementsByClassName('canvas-copy');
			for (var i = 0; i < srcCanvas.length; i++)
			{
				printImg[i].src = srcCanvas[i].toDataURL();
			}
		};

	var url = 'print_template.jsp?ID='+ID+'&callback=finalizePrint';
	if (extraStyles) url += '&extraStyles='+extraStyles;
	if (reggieRoot) url = reggieRoot+url;
	// Open the print-window 
	var printWin = window.open(url, 'PrintWindow', 'width='+width+',height='+height+',toolbar=yes,location=no,directories=no,status=no,menubar=yes,scrollbars=yes,resizable=yes');

}

/*
	Set the current working directory for the logged in user to the given path.
	If the given path is not found, the working directory is not changed.
	
	@param ID The session id
	@param path The path to set
	@param debug TRUE to reduce time between AJAX calls and print debug information
	@param reggieRoot Optional relative path to the root of the Reggie installation (for pages in a subdirectory)
*/
function setCurrentDirectory(ID, path, debug, reggieRoot)
{
	var root = reggieRoot ? reggieRoot : '';
	var url = root + 'Session.servlet?ID='+ID+'&cmd=SetCurrentDirectory&path='+encodeURIComponent(path);
	var request = Main.getAjaxRequest();
	request.open("GET", url, false);
	request.send(null);
	if (debug) Main.debug(request.responseText);
	var response = JSON.parse(request.responseText);
	return response.status == 'ok' ? response.path : null;
}

function getSubtypeInfo(ID, subtypeName, debug, reggieRoot)
{
	var root = reggieRoot ? reggieRoot : '';
	var url = root + 'Subtype.servlet?ID='+ID+'&cmd=GetSubtypeInfo';
	url += '&name='+subtypeName;
	
	var request = Ajax.getXmlHttpRequest();
	request.open("GET", url, false);
	request.send(null);

	if (debug) Main.debug(request.responseText);
	var response = JSON.parse(request.responseText);
	return response.status == 'ok' ? response.subtype : null;
}

/*
	Call this method once to start a repeating timer that will submit
	an Ajax request at regular intervals in order to keep the current login
	session alive. Once started there is no way to stop the timer except
	navigating away from the page.
	@param ID The session id
	@param debug TRUE to reduce time between AJAX calls and print debug information
	@param reggieRoot Optional relative path to the root of the Reggie installation (for pages in a subdirectory)
*/
function keepSessionAlive(ID, debug, reggieRoot)
{
	var internal = {};
	internal.h1 = document.getElementsByTagName('h1')[0];
	internal.interval = debug ? 5000 : 30000;
	internal.root = reggieRoot ? reggieRoot : '';
	
	if (debug)
	{
		// Create a 'simulate timeout' checkbox
		var div = document.createElement('div');
		div.style.textAlign = 'center';
		div.innerHTML = '<input type="checkbox" id="simulateTimeout"><label for="simulateTimeout">Simulate session timeout</label>';
		var frm = document.forms['reggie'];
		frm.appendChild(div);
	}
	
	internal.sendKeepAlive = function()
	{
		Main.addClass(internal.h1, 'working');
		var simulateTimeout = false;
		if (debug) 
		{
			var msg = 'checking if session is alive...';
			var timeoutDiv = document.getElementById('simulateTimeout');
			if (timeoutDiv && timeoutDiv.checked)
			{
				simulateTimeout = true;
				timeoutDiv.checked = false;
				msg = 'simulating a session timeout...'
			}
			Main.debug(msg);
		}
		
		var url = internal.root + 'Session.servlet?ID='+ID+'&cmd=CheckSession';
		if (simulateTimeout) url += '&simulateTimeout=1';
		var request = Main.getAjaxRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, internal.onKeepAliveResponse);
		request.send(null);
		
		setTimeout(internal.repeatAgain, 1000);	
	}
	
	internal.repeatAgain = function()
	{
		Main.removeClass(internal.h1, 'working');
		setTimeout(internal.sendKeepAlive, internal.interval);
	}
	
	internal.onKeepAliveResponse = function(request)
	{
		if (debug) Main.debug(request.responseText);
		var response = JSON.parse(request.responseText);	
		if (response.status != 'ok')
		{
			setFatalError(response.message);
			return false;
		}
		if (!response.session)
		{
			var keepAlive = Main.getPageValue('open.windows', 'KeepSessionAlive');
			if (!keepAlive || keepAlive.closed)
			{
				Main.openPopup(internal.root+'keep_session_alive.jsp?ID='+ID, 'KeepSessionAlive', 800, 600);
			}
		}
	}
	
	internal.repeatAgain();
}